import requests
import os
import json
import logging
import pandas as pd
from datetime import datetime, timedelta
import time
import threading

# Setup logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

class KoreaInvestmentAPI:
    def __init__(self):
        self.appkey = os.getenv('MY_APP')
        self.appsecret = os.getenv('MY_SEC')
        self.user_id = os.getenv('USER_ID')
        self.base_url = os.getenv('PROD_URL')

        token_path = os.path.join('C:\\Users\\user\\KIS\\config', 'token.json')
        if not os.path.exists(token_path):
            raise ValueError(f"Token file does not exist: {token_path}")

        with open(token_path, 'r') as file:
            token_data = json.load(file)
            self.access_token = token_data.get('token')

        if not all([self.appkey, self.appsecret, self.access_token, self.user_id, self.base_url]):
            missing_vars = [var for var in ['MY_APP', 'MY_SEC', 'ACCESS_TOKEN', 'USER_ID', 'PROD_URL'] if os.getenv(var) is None]
            raise ValueError(f"Some required environment variables are missing: {', '.join(missing_vars)}")

    def get_headers(self, tr_id):
        return {
            "Content-Type": "application/json; charset=utf-8",
            "authorization": f"Bearer {self.access_token}",
            "appkey": self.appkey,
            "appsecret": self.appsecret,
            "tr_id": tr_id,
            "custtype": "P"
        }

    def get_search_conditions(self):
        path = "uapi/domestic-stock/v1/quotations/psearch-title"
        url = f"{self.base_url}/{path}"
        headers = self.get_headers("HHKST03900300")
        params = {"user_id": self.user_id}

        try:
            logging.debug(f"Making GET request to {url} with headers {headers} and params {params}")
            response = requests.get(url, headers=headers, params=params, timeout=10)
            response.raise_for_status()
            logging.info("Search conditions response: %s", response.json())
            return response.json().get('output1', [])
        except requests.exceptions.Timeout:
            logging.error("Request timed out")
        except requests.exceptions.HTTPError as http_err:
            logging.error(f"HTTP error occurred: {http_err}")
            logging.error(f"Response content: {response.content}")
        except Exception as err:
            logging.error(f"Other error occurred: {err}")

        return []

    def get_historical_data(self, stock_code, start_date, end_date):
        path = "uapi/domestic-stock/v1/quotations/inquire-daily-itemchartprice"
        url = f"{self.base_url}/{path}"
        headers = self.get_headers("FHKST03010100")
        params = {
            "FID_COND_MRKT_DIV_CODE": "J",
            "FID_INPUT_ISCD": stock_code,
            "FID_INPUT_DATE_1": start_date,
            "FID_INPUT_DATE_2": end_date,
            "FID_PERIOD_DIV_CODE": "D",
            "FID_ORG_ADJ_PRC": "0"
        }

        try:
            logging.debug(f"Making GET request to {url} with headers {headers} and params {params}")
            response = requests.get(url, headers=headers, params=params, timeout=10)
            response.raise_for_status()
            data = response.json()
            logging.info("Historical data response: %s", data)
            return data.get('output2', [])
        except requests.exceptions.Timeout:
            logging.error("Request timed out")
        except requests.exceptions.HTTPError as http_err:
            logging.error(f"HTTP error occurred: {http_err}")
            logging.error(f"Response content: {response.content}")
        except Exception as err:
            logging.error(f"Other error occurred: {err}")

        return []

    def get_minute_data(self, stock_code):
        path = "uapi/domestic-stock/v1/quotations/inquire-time-itemchartprice"
        url = f"{self.base_url}/{path}"
        headers = self.get_headers("FHKST03010200")
        current_time = datetime.now().strftime('%H%M')
        params = {
            "FID_COND_MRKT_DIV_CODE": "J",
            "FID_INPUT_ISCD": stock_code,
            "FID_INPUT_HOUR_1": current_time
        }

        try:
            logging.debug(f"Making GET request to {url} with headers {headers} and params {params}")
            response = requests.get(url, headers=headers, params=params, timeout=10)
            response.raise_for_status()
            data = response.json()
            logging.info("Minute data response: %s", data)
            return data.get('output2', [])
        except requests.exceptions.Timeout:
            logging.error("Request timed out")
        except requests.exceptions.HTTPError as http_err:
            logging.error(f"HTTP error occurred: {http_err}")
            logging.error(f"Response content: {response.content}")
        except Exception as err:
            logging.error(f"Other error occurred: {err}")

        return []

    def accumulate_historical_data(self):
        # Get stock list from search conditions
        stocks = self.get_search_conditions()

        # Determine date range
        end_date = datetime.today().strftime('%Y%m%d')
        start_date = (datetime.today() - timedelta(days=5*365)).strftime('%Y%m%d')

        # Dictionary to store accumulated data
        accumulated_data = {}

        for stock in stocks:
            stock_code = stock['stck_shrn_iscd']
            historical_data = self.get_historical_data(stock_code, start_date, end_date)
            if stock_code in accumulated_data:
                accumulated_data[stock_code].extend(historical_data)
            else:
                accumulated_data[stock_code] = historical_data

        # Save accumulated data to a CSV file
        for stock_code, data in accumulated_data.items():
            df = pd.DataFrame(data)
            numeric_columns = ['stck_clpr', 'stck_oprc', 'stck_hgpr', 'stck_lwpr', 'acml_vol', 'acml_tr_pbmn', 'prdy_vrss']
            df[numeric_columns] = df[numeric_columns].apply(pd.to_numeric)
            df['stck_bsop_date'] = pd.to_datetime(df['stck_bsop_date'])
            df.to_csv(f'{stock_code}_historical_data.csv', index=False)

        return accumulated_data

    def accumulate_minute_data(self):
        # Get stock list from search conditions
        stocks = self.get_search_conditions()

        # Initialize storage for minute data
        minute_data_storage = {}

        while True:
            for stock in stocks:
                stock_code = stock['stck_shrn_iscd']
                minute_data = self.get_minute_data(stock_code)
                if stock_code in minute_data_storage:
                    minute_data_storage[stock_code].extend(minute_data)
                else:
                    minute_data_storage[stock_code] = minute_data

                # Save accumulated minute data to a CSV file
                df = pd.DataFrame(minute_data_storage[stock_code])
                numeric_columns = ['stck_prpr', 'acml_vol', 'acml_tr_pbmn']
                df[numeric_columns] = df[numeric_columns].apply(pd.to_numeric)
                df['stck_bsop_date'] = pd.to_datetime(df['stck_bsop_date'])
                df.to_csv(f'{stock_code}_minute_data.csv', index=False)

            # Sleep for a minute before fetching the next set of minute data
            time.sleep(60)

    def schedule_historical_data_collection(self):
        while True:
            current_time = datetime.now()
            if current_time.hour >= 15:  # Assuming market closes at 3 PM
                logging.info("Market closed, collecting historical data...")
                self.accumulate_historical_data()
                # Sleep until the next day
                time.sleep(24 * 60 * 60)
            else:
                # Sleep for a while before checking the time again
                time.sleep(60)

def main():
    api = KoreaInvestmentAPI()

    # Start minute-by-minute data accumulation
    minute_data_thread = threading.Thread(target=api.accumulate_minute_data)
    minute_data_thread.start()

    # Schedule historical data collection after market closure
    api.schedule_historical_data_collection()

if __name__ == "__main__":
    main()
