import os
import json
import requests
import logging
import pandas as pd
import time
from datetime import datetime, timedelta
from dotenv import load_dotenv
from kis_api import fetch_stock_data

# Load environment variables
load_dotenv('C:\\Users\\user\\projects\\st\\.env')

# Logging configuration
logging.basicConfig(level=logging.INFO)

# Define settings dictionary
settings = {
    'token_path': os.getenv('TOKEN_PATH'),
    'appkey': os.getenv('MY_APP'),
    'appsecret': os.getenv('MY_SEC'),
    'user_id': os.getenv('USER_ID'),
    'data_path': os.getenv('DATA_PATH'),
    'model_path': os.getenv('MODEL_PATH'),
    'trading_url': os.getenv('PROD_URL')
}

def get_token_data():
    token_path = settings['token_path']
    try:
        with open(token_path, 'r') as file:
            return json.load(file)
    except (FileNotFoundError, json.JSONDecodeError) as e:
        logging.error(f"Error reading token file: {e}")
        return None

def save_token_data(token_data):
    token_path = settings['token_path']
    with open(token_path, 'w') as file:
        json.dump(token_data, file)

def is_token_valid(token_data):
    if not token_data or 'access_token' not in token_data:
        return False
    expiration_str = token_data.get('expiration')
    if not expiration_str:
        return True
    expiration = datetime.strptime(expiration_str, "%Y-%m-%d %H:%M:%S")
    return expiration > datetime.now()

def get_valid_token():
    token_data = get_token_data()
    if is_token_valid(token_data):
        logging.info("Using existing token")
        return token_data['access_token']
    else:
        logging.info("Token expired or invalid, refreshing...")
        return refresh_token()

def refresh_token(retries=3, delay=60):
    url = "https://openapi.koreainvestment.com:9443/oauth2/tokenP"
    data = {
        'grant_type': 'client_credentials',
        'appkey': settings['appkey'],
        'appsecret': settings['appsecret']
    }
    headers = {
        'Content-Type': 'application/json'
    }
    
    for attempt in range(retries):
        try:
            logging.info("Requesting new token...")
            response = requests.post(url, headers=headers, json=data)
            if response.status_code == 200:
                response_data = response.json()
                new_token = response_data.get('access_token')
                expires_in = response_data.get('access_token_token_expired')
                logging.info(f"expires_in value: {expires_in}")
                try:
                    if expires_in:
                        expiration = datetime.now() + timedelta(seconds=int(expires_in))
                    else:
                        expiration = datetime.now() + timedelta(hours=1)
                except ValueError as e:
                    logging.error(f"Error parsing expires_in value: {e}")
                    expiration = datetime.now() + timedelta(hours=1)
                save_token_data({'access_token': new_token, 'expiration': expiration.strftime("%Y-%m-%d %H:%M:%S")})
                logging.info(f"New token saved. Expiration time: {expiration}")
                return new_token
            else:
                logging.error(f"Error refreshing token: {response.text}")
                if "EGW00133" in response.text:
                    logging.info(f"Retrying in {delay} seconds...")
                    time.sleep(delay)
                else:
                    return None
        except requests.exceptions.RequestException as e:
            logging.error(f"Request exception occurred while refreshing token: {e}")
            return None
    return None

def get_api_response(url, headers, params):
    try:
        response = requests.get(url, headers=headers, params=params)
        response.raise_for_status()
        return response.json()
    except requests.exceptions.RequestException as e:
        logging.error(f"Request exception occurred: {e}")
        return None

def get_search_conditions():
    access_token = get_valid_token()
    if not access_token:
        logging.error("Failed to obtain access token.")
        return None

    url = "https://openapi.koreainvestment.com:9443/uapi/domestic-stock/v1/quotations/psearch-title"
    headers = {
        'Content-Type': 'application/json; charset=utf-8',
        'Authorization': f'Bearer {access_token}',
        'appkey': settings['appkey'],
        'appsecret': settings['appsecret'],
        'tr_id': 'HHKST03900300'
    }
    params = {
        'user_id': settings['user_id']
    }

    logging.info(f"Request URL: {url}")
    logging.info(f"Request Headers: {headers}")
    logging.info(f"Request Params: {params}")

    return get_api_response(url, headers, params)

def get_search_results(seq):
    access_token = get_valid_token()
    if not access_token:
        logging.error("Failed to obtain access token.")
        return None

    url = "https://openapi.koreainvestment.com:9443/uapi/domestic-stock/v1/quotations/psearch-result"
    headers = {
        'Content-Type': 'application/json; charset=utf-8',
        'Authorization': f'Bearer {access_token}',
        'appkey': settings['appkey'],
        'appsecret': settings['appsecret'],
        'tr_id': 'HHKST03900400'
    }
    params = {
        'user_id': settings['user_id'],
        'seq': seq
    }

    response = get_api_response(url, headers, params)
    if response:
        # Filter only stock items
        stock_results = []
        for result in response.get('output2', []):
            if result.get('daebi') == '2' and '인버스' not in result.get('name', '') and 'ETF' not in result.get('name', ''):
                stock_results.append(result)
        return {'output2': stock_results}
    return None

def save_data_to_csv(data, stock_code):
    file_path = os.path.join(settings['data_path'], f"{stock_code}_data.csv")

    if isinstance(data, dict):
        df = pd.DataFrame([data])
        required_columns = ['Open', 'High', 'Low', 'Close', 'Volume', 'position']
        
        # 기본값 추가 전에 실제 값으로 설정하기
        if 'Open' not in df.columns:
            df['Open'] = df['askp1']
        if 'High' not in df.columns:
            df['High'] = df['askp1'] + 10  # 임의의 값 설정
        if 'Low' not in df.columns:
            df['Low'] = df['bidp1'] - 10  # 임의의 값 설정
        if 'Close' not in df.columns:
            df['Close'] = df['askp1']
        if 'Volume' not in df.columns:
            df['Volume'] = 1000  # 임의의 값 설정
        if 'position' not in df.columns:
            df['position'] = 0
        
        df.to_csv(file_path, index=False)
        logging.info(f"Data for stock {stock_code} saved to {file_path}")
    else:
        logging.error(f"Data format for stock {stock_code} is not suitable for CSV: {data}")

def process_stock_data(stock):
    stock_code = stock.get('code')
    stock_data = fetch_stock_data(
        stock_code,
        settings['appkey'],
        settings['appsecret'],
        get_valid_token()
    )
    
    logging.info(f"Fetched data for stock {stock_code}: {stock_data}")
    
    if stock_data is not None:
        save_data_to_csv(stock_data, stock_code)
    else:
        logging.error(f"Failed to fetch data for stock {stock_code}")

def collect_condition_data(stock_code=None):
    try:
        if stock_code:
            process_stock_data({'code': stock_code})
        else:
            search_conditions = get_search_conditions()
            if not search_conditions or 'output2' not in search_conditions:
                logging.error("No search conditions found.")
                return

            for condition in search_conditions['output2']:
                seq = condition.get('seq')
                search_results = get_search_results(seq)
                if not search_results or 'output2' not in search_results:
                    logging.error(f"No search results found for condition sequence {seq}.")
                    continue

                for stock in search_results['output2']:
                    process_stock_data(stock)
    except Exception as e:
        logging.error(f"Error collecting condition data: {e}")

if __name__ == "__main__":
    collect_condition_data()
