import os
import json
import requests
from dotenv import load_dotenv
import logging
from datetime import datetime, timedelta

# Load environment variables from .env file
load_dotenv('C:\\Users\\user\\projects\\st\\.env')

logging.basicConfig(level=logging.INFO)

def get_valid_token():
    token_path = os.getenv('TOKEN_PATH')
    logging.info(f"Checking token at {token_path}")

    try:
        with open(token_path, 'r') as file:
            token_data = json.load(file)
        access_token = token_data.get('access_token')
        expiration_str = token_data.get('expiration')
        if expiration_str:
            expiration = datetime.strptime(expiration_str, "%Y-%m-%d %H:%M:%S")
            logging.info(f"Token expiration time: {expiration}")
            if expiration > datetime.now():
                logging.info("Using existing token")
                return access_token
            else:
                logging.info("Token expired, refreshing...")
                return refresh_token()
        else:
            logging.info("Expiration time not found in token data, using existing token")
            return access_token if access_token else refresh_token()
    except (FileNotFoundError, json.JSONDecodeError, KeyError, TypeError) as e:
        logging.error(f"Error loading token: {e}")
        return refresh_token()

def refresh_token(retries=3, delay=60):
    url = "https://openapi.koreainvestment.com:9443/oauth2/tokenP"
    data = {
        'grant_type': 'client_credentials',
        'appkey': os.getenv('MY_APP'),
        'appsecret': os.getenv('MY_SEC')
    }
    headers = {
        'Content-Type': 'application/json'
    }
    
    for attempt in range(retries):
        try:
            logging.info("Requesting new token...")
            response = requests.post(url, headers=headers, json=data)
            if response.status_code == 200:
                response_data = response.json()
                new_token = response_data.get('access_token')
                expires_in = response_data.get('access_token_token_expired')
                logging.info(f"expires_in value: {expires_in}")  # expires_in 값을 로그로 출력
                try:
                    if expires_in:
                        expiration = datetime.now() + timedelta(seconds=int(expires_in))
                    else:
                        expiration = datetime.now() + timedelta(hours=1)  # 만약 expires_in이 없을 경우 1시간 유효기간 설정
                except ValueError as e:
                    logging.error(f"Error parsing expires_in value: {e}")
                    expiration = datetime.now() + timedelta(hours=1)  # 예외 발생 시 1시간 유효기간 설정
                token_path = os.getenv('TOKEN_PATH')
                with open(token_path, 'w') as file:
                    json.dump({'access_token': new_token, 'expiration': expiration.strftime("%Y-%m-%d %H:%M:%S")}, file)
                logging.info(f"New token saved. Expiration time: {expiration}")
                return new_token
            else:
                logging.error(f"Error refreshing token: {response.text}")
                if "EGW00133" in response.text:
                    logging.info(f"Retrying in {delay} seconds...")
                    time.sleep(delay)
                else:
                    return None
        except requests.exceptions.RequestException as e:
            logging.error(f"Request exception occurred while refreshing token: {e}")
            return None
    return None

def get_search_conditions():
    access_token = get_valid_token()
    if not access_token:
        logging.error("Failed to obtain access token.")
        return None

    url = "https://openapi.koreainvestment.com:9443/uapi/domestic-stock/v1/quotations/psearch-title"
    headers = {
        'Content-Type': 'application/json; charset=utf-8',
        'Authorization': f'Bearer {access_token}',
        'appkey': os.getenv('MY_APP'),
        'appsecret': os.getenv('MY_SEC'),
        'tr_id': 'HHKST03900300'
    }
    params = {
        'user_id': os.getenv('USER_ID')
    }

    logging.info(f"Request URL: {url}")
    logging.info(f"Request Headers: {headers}")
    logging.info(f"Request Params: {params}")

    try:
        response = requests.get(url, headers=headers, params=params)
        if response.status_code == 200:
            data = response.json()
            logging.info(f"Search conditions response: {data}")
            return data['output2']
        else:
            logging.error(f"Error fetching search conditions: {response.text}")
            return None
    except requests.exceptions.RequestException as e:
        logging.error(f"Request exception occurred: {e}")
        return None

def get_search_results(seq):
    access_token = get_valid_token()
    if not access_token:
        logging.error("Failed to obtain access token.")
        return None

    url = "https://openapi.koreainvestment.com:9443/uapi/domestic-stock/v1/quotations/psearch-result"
    headers = {
        'Content-Type': 'application/json; charset=utf-8',
        'Authorization': f'Bearer {access_token}',
        'appkey': os.getenv('MY_APP'),
        'appsecret': os.getenv('MY_SEC'),
        'tr_id': 'HHKST03900400'
    }
    params = {
        'user_id': os.getenv('USER_ID'),
        'seq': seq
    }

    logging.info(f"Request URL: {url}")
    logging.info(f"Request Headers: {headers}")
    logging.info(f"Request Params: {params}")

    try:
        response = requests.get(url, headers=headers, params=params)
        if response.status_code == 200:
            data = response.json()
            logging.info(f"Search results response: {data}")
            return data['output2']
        else:
            logging.error(f"Error fetching search results: {response.text}")
            return None
    except requests.exceptions.RequestException as e:
        logging.error(f"Request exception occurred: {e}")
        return None

def get_stock_codes():
    search_conditions = get_search_conditions()
    if not search_conditions:
        return []

    stock_codes = []
    for condition in search_conditions:
        seq = condition['seq']
        search_results = get_search_results(seq)
        if search_results:
            for result in search_results:
                stock_codes.append(result['code'])

    return stock_codes
