
import pandas as pd
import yfinance as yf
import numpy as np
import gym
from stable_baselines3 import PPO
from stable_baselines3.common.vec_env import DummyVecEnv
from sklearn.preprocessing import StandardScaler

# Function to load stock codes from file
def load_stock_codes(file_path):
    try:
        stock_codes = pd.read_csv(file_path, delimiter='|')
        return stock_codes
    except Exception as e:
        print(f"Error loading stock codes: {e}")
        return None

# Load KOSDAQ and KOSPI stock codes
kosdaq_file_path = '/mnt/data/modules/config/kosdaq_code.mst'
kospi_file_path = '/mnt/data/modules/config/kospi_code.mst'

kosdaq_codes = load_stock_codes(kosdaq_file_path)
kospi_codes = load_stock_codes(kospi_file_path)

# Function to fetch stock data
def fetch_stock_data(stock_code, start_date, end_date):
    try:
        stock_data = yf.download(stock_code, start=start_date, end=end_date)
        return stock_data
    except Exception as e:
        print(f"Error fetching data for {stock_code}: {e}")
        return None

# Example stock code and date range
example_stock_code = 'AAPL'
start_date = '2023-01-01'
end_date = '2023-12-31'

# Fetch stock data
stock_data = fetch_stock_data(example_stock_code, start_date, end_date)
if stock_data is not None:
    print(stock_data.head())
else:
    print("Failed to fetch stock data")

# Define the stock trading environment for reinforcement learning
class StockTradingEnv(gym.Env):
    def __init__(self, stock_data):
        super(StockTradingEnv, self).__init__()
        self.stock_data = stock_data
        self.scaler = StandardScaler()
        self.state = self.scaler.fit_transform(stock_data)
        self.action_space = gym.spaces.Discrete(3)  # Buy, Hold, Sell
        self.observation_space = gym.spaces.Box(low=-np.inf, high=np.inf, shape=(self.state.shape[1],))
        self.current_step = 0
    
    def step(self, action):
        reward = 0
        self.current_step += 1
        done = self.current_step >= len(self.stock_data) - 1
        if action == 0:  # Buy
            reward = 1
        elif action == 2:  # Sell
            reward = -1
        return self.state[self.current_step], reward, done, {}
    
    def reset(self):
        self.current_step = 0
        return self.state[self.current_step]

# Process and scale the stock data for training
if stock_data is not None and not stock_data.empty:
    # Use only the 'Close' prices for simplicity
    stock_data = stock_data[['Close']]
    scaled_data = StandardScaler().fit_transform(stock_data)
    env = DummyVecEnv([lambda: StockTradingEnv(scaled_data)])
    
    # Train the reinforcement learning model
    model = PPO('MlpPolicy', env, verbose=1)
    model.learn(total_timesteps=10000)

    # Use the model to recommend an action
    obs = env.reset()
    for i in range(len(scaled_data)):
        action, _ = model.predict(obs)
        obs, rewards, done, info = env.step(action)
        if done:
            break

    print(f"Recommended action: {['Buy', 'Hold', 'Sell'][action]}")
else:
    print("No valid stock data available for training.")
