import os
import json
import requests
from dotenv import load_dotenv

# Load environment variables
load_dotenv('C:\\Users\\user\\projects\\st\\.env')

# Settings dictionary
settings = {
    'data_path': os.getenv('DATA_PATH', 'data'),
    'token_path': os.getenv('TOKEN_PATH'),
    'appkey': os.getenv('MY_APP'),
    'appsecret': os.getenv('MY_SEC'),
    'user_id': os.getenv('USER_ID'),
    'trading_url': os.getenv('PROD_URL'),  # 실제 URL을 넣어주세요
    'agent': os.getenv('MY_AGENT'),
    'acct_stock': os.getenv('MY_ACCT_STOCK'),
    'cano': os.getenv('CANO')
}

def read_token(token_path):
    try:
        if not os.path.exists(token_path):
            print(f"Token file not found at {token_path}")
            return None

        with open(token_path, 'r') as file:
            token_data = json.load(file)
            print(f"Token data: {token_data}")  # Debugging: print token data
            token = token_data.get('access_token')  # Update to access_token
            return token
    except FileNotFoundError:
        print(f"Token file not found at {token_path}")
        return None
    except json.JSONDecodeError:
        print("Error decoding the token JSON file.")
        return None

def get_stock_lower_price(stock_code):
    access_token = read_token(settings['token_path'])
    if not access_token:
        print("Access token is missing.")
        return None

    url = f"{settings['trading_url']}/uapi/domestic-stock/v1/quotations/inquire-price"
    headers = {
        "Content-Type": "application/json; charset=utf-8",
        "Authorization": f"Bearer {access_token}",
        "appkey": settings['appkey'],
        "appsecret": settings['appsecret'],
        "tr_id": "FHKST01010100",  # 주식 현재가 조회
        "User-Agent": settings['agent']
    }

    params = {
        "FID_COND_MRKT_DIV_CODE": "J",
        "FID_INPUT_ISCD": stock_code
    }

    try:
        response = requests.get(url, headers=headers, params=params)
        response_data = response.json()
        print(f"API response: {json.dumps(response_data, indent=4)}")  # Print the entire response for debugging
        if response.status_code == 200 and response_data.get('rt_cd') == '0':
            lower_price = int(response_data['output']['stck_llam'])  # 하한가
            print(f"Lower price for stock {stock_code}: {lower_price}")
            return lower_price
        else:
            print(f"Failed to get lower price: {response_data}")
            return None
    except requests.exceptions.RequestException as e:
        print(f"Request exception occurred: {e}")
        return None

def order_stock(stock_code, quantity, price, order_type="00"):
    access_token = read_token(settings['token_path'])
    if not access_token:
        print("Access token is missing.")
        return None

    url = f"{settings['trading_url']}/uapi/domestic-stock/v1/trading/order-cash"
    headers = {
        "Content-Type": "application/json; charset=utf-8",
        "Authorization": f"Bearer {access_token}",
        "appkey": settings['appkey'],
        "appsecret": settings['appsecret'],
        "tr_id": "TTTC0802U",  # 현금 매수 주문
        "User-Agent": settings['agent']
    }

    data = {
        "CANO": settings['cano'],
        "ACNT_PRDT_CD": settings['acct_stock'],
        "PDNO": stock_code,
        "ORD_DVSN": order_type,
        "ORD_QTY": str(quantity),
        "ORD_UNPR": str(price)
    }

    try:
        response = requests.post(url, headers=headers, json=data)
        response_data = response.json()
        if response.status_code == 200 and response_data.get('rt_cd') == '0':
            print(f"Order successful: {response_data}")
            return response_data
        else:
            print(f"Order failed: {response_data}")
            return None
    except requests.exceptions.RequestException as e:
        print(f"Request exception occurred: {e}")
        return None

if __name__ == "__main__":
    # Example usage: order 10 shares of stock code '005930'
    stock_code = '005930'
    quantity = 10

    lower_price = get_stock_lower_price(stock_code)
    if lower_price:
        # Set the order price above the lower limit
        order_price = lower_price + 100  # Set 100 won above the lower limit
        result = order_stock(stock_code, quantity, order_price)
        if result:
            print(f"Order result: {result}")
        else:
            print("Order failed.")
    else:
        print("Failed to get the lower price for the stock.")
