import os
import json
import requests
import pandas as pd
from dotenv import load_dotenv

# Load environment variables from .env file
load_dotenv('C:\\Users\\user\\projects\\st\\.env')

# Settings dictionary
settings = {
    'data_path': os.getenv('DATA_PATH', 'data'),
    'model_path': os.getenv('MODEL_PATH', 'models'),
    'approval_key': os.getenv('APPROVAL_KEY'),
    'max_detected_stocks': int(os.getenv('MAX_DETECTED_STOCKS', 99)),
    'data_fetch_interval': int(os.getenv('DATA_FETCH_INTERVAL', 60)),
    'strategy_name': os.getenv('STRATEGY_NAME', 'AdvancedAdaptiveStrategy'),
    'token_path': os.getenv('TOKEN_PATH', 'C:\\Users\\user\\KIS\\config\\token.json'),
    'appkey': os.getenv('MY_APP'),
    'appsecret': os.getenv('MY_SEC'),
    'user_id': os.getenv('USER_ID'),
    'trading_url': os.getenv('PROD_URL'),
    'agent': os.getenv('MY_AGENT'),
    'acct_stock': os.getenv('MY_ACCT_STOCK'),
    'prod': os.getenv('MY_PROD'),
    'cano': os.getenv('CANO')
}

def read_token(token_path):
    try:
        if not os.path.exists(token_path):
            print(f"Token file not found at {token_path}")
            return None
        
        with open(token_path, 'r') as file:
            token_data = json.load(file)
            print(f"Token data: {token_data}")  # Debugging: print token data
            token = token_data.get('access_token')  # Update to access_token
            return token
    except FileNotFoundError:
        print(f"Token file not found at {token_path}")
        return None
    except json.JSONDecodeError:
        print("Error decoding the token JSON file.")
        return None

def get_acct_balance():
    access_token = read_token(settings['token_path'])
    if not access_token:
        print("Access token is missing.")
        return None

    url = f"{settings['trading_url']}/uapi/domestic-stock/v1/trading/inquire-balance"
    headers = {
        "Content-Type": "application/json; charset=utf-8",
        "Authorization": f"Bearer {access_token}",
        "appkey": settings['appkey'],
        "appsecret": settings['appsecret'],
        "tr_id": "TTTC8434R",
        "User-Agent": settings['agent']
    }

    params = {
        "CANO": settings['cano'],
        "ACNT_PRDT_CD": settings['acct_stock'],
        "AFHR_FLPR_YN": "N",
        "OFL_YN": "",
        "INQR_DVSN": "01",
        "UNPR_DVSN": "01",
        "FUND_STTL_ICLD_YN": "N",
        "FNCG_AMT_AUTO_RDPT_YN": "N",
        "PRCS_DVSN": "00",
        "CTX_AREA_FK100": "",
        "CTX_AREA_NK100": ""
    }

    try:
        response = requests.get(url, headers=headers, params=params)
        if response.status_code == 200:
            data = response.json()
            print(f"API response: {json.dumps(data, indent=4)}")  # Print the entire response for debugging
            if data['rt_cd'] == '0':
                if 'output1' in data:
                    df = pd.DataFrame(data['output1'])
                    # Check if 'output2' is present and add 'ord_psbl_cash'
                    if 'output2' in data and len(data['output2']) > 0 and 'ord_psbl_cash' in data['output2'][0]:
                        df['ord_psbl_cash'] = data['output2'][0]['ord_psbl_cash']
                    else:
                        print("ord_psbl_cash not found in output2.")
                    
                    df.to_csv(os.path.join(settings['data_path'], 'account_balance.csv'), index=False)  # Save to CSV file
                    # Save to JSON file
                    balance_json_path = os.path.join(settings['data_path'], 'balance.json')
                    df.to_json(balance_json_path, orient='records', force_ascii=False)
                    print(f"Account balance saved to {balance_json_path}")
                    return df
                else:
                    print("No data found in API response.")
                    return pd.DataFrame()
            else:
                print("API error:", data['msg1'])
                return None
        else:
            print("HTTP error:", response.status_code)
            print("Response content:", response.content)
            return None
    except requests.exceptions.RequestException as e:
        print("Request exception occurred:", e)
        return None

if __name__ == "__main__":
    balance_df = get_acct_balance()
    if balance_df is not None:
        print(balance_df)
    else:
        print("잔액을 조회할 수 없습니다.")
