import backtrader as bt
import numpy as np
import pandas as pd
from sklearn.ensemble import RandomForestClassifier
import logging

# Setup logging
logging.basicConfig(level=logging.INFO)

class VWAP(bt.Indicator):
    lines = ('vwap',)
    params = (('period', 15),)

    def __init__(self):
        self.addminperiod(self.params.period)
        self.plotinfo.plotyhlines = [0.0]

    def next(self):
        typical_price = (self.data.close[0] + self.data.high[0] + self.data.low[0]) / 3
        if len(self) == 1:
            self.lines.vwap[0] = typical_price
        else:
            cumulative_price_volume = (self.lines.vwap[-1] * (len(self) - 1) + typical_price * self.data.volume[0])
            cumulative_volume = (len(self) - 1) * self.data.volume[0] + self.data.volume[0]
            self.lines.vwap[0] = cumulative_price_volume / cumulative_volume

class AdvancedAdaptiveStrategy(bt.Strategy):
    params = (
        ('lookback', 15),  
        ('rsi_period', 14),  
        ('macd1', 12),  
        ('macd2', 26),  
        ('macdsig', 9),  
        ('rsi_overbought', 70),  
        ('rsi_oversold', 30),  
        ('stop_loss', 0.02),  
        ('take_profit', 0.04),  
        ('volatility_window', 20)  
    )

    def __init__(self):
        self.vwap = VWAP(self.data, period=self.params.lookback)
        self.rsi = bt.indicators.RelativeStrengthIndex(self.data, period=self.params.rsi_period)
        self.macd = bt.indicators.MACD(self.data,
                                       period_me1=self.params.macd1,
                                       period_me2=self.params.macd2,
                                       period_signal=self.params.macdsig)
        self.bollinger = bt.indicators.BollingerBands(self.data)
        self.atr = bt.indicators.AverageTrueRange(self.data)
        
        self.model = RandomForestClassifier()
        self.train_model()

    def train_model(self):
        """
        Train the machine learning model using historical data and various indicators.
        """
        data = self._prepare_data()
        features = self._generate_features(data)
        labels = self._generate_labels(data)

        if features.empty or labels.empty:
            logging.error("Features or labels are empty. Cannot train the model.")
            return

        logging.info("Training the model with prepared features and labels.")
        self.model.fit(features, labels)

    def _prepare_data(self) -> pd.DataFrame:
        """
        Prepare the historical data for feature generation.
        
        :return: DataFrame with historical data.
        """
        data = pd.DataFrame({
            'close': [self.data.close[i] for i in range(len(self.data))],
            'volume': [self.data.volume[i] for i in range(len(self.data))],
            'high': [self.data.high[i] for i in range(len(self.data))],
            'low': [self.data.low[i] for i in range(len(self.data))]
        })
        return data

    def _generate_features(self, data: pd.DataFrame) -> pd.DataFrame:
        """
        Generate features for the machine learning model.
        
        :param data: DataFrame with historical data.
        :return: DataFrame with generated features.
        """
        features = data[['close', 'volume', 'high', 'low']].pct_change()
        features['vwap'] = self._calculate_vwap(data)
        features['rsi'] = data['close'].rolling(window=self.params.rsi_period).apply(self._rsi_calculate)
        features['macd'] = self._calculate_macd(data)
        features['bollinger_high'], features['bollinger_low'] = self._calculate_bollinger_bands(data)
        features = features.dropna()
        logging.info(f"Features after dropping NaNs:\n{features}")
        return features

    def _generate_labels(self, data: pd.DataFrame) -> pd.Series:
        """
        Generate labels for the machine learning model.
        
        :param data: DataFrame with historical data.
        :return: Series with labels.
        """
        labels = (data['close'].shift(-1) > data['close']).astype(int)
        logging.info(f"Labels after alignment:\n{labels}")
        return labels

    def _calculate_vwap(self, data: pd.DataFrame) -> pd.Series:
        typical_price = (data['close'] + data['high'] + data['low']) / 3
        return (typical_price * data['volume']).cumsum() / data['volume'].cumsum()

    def _rsi_calculate(self, series: pd.Series) -> float:
        delta = series.diff()
        gain = (delta.where(delta > 0, 0)).mean()
        loss = (-delta.where(delta < 0, 0)).mean()
        rs = gain / loss
        return 100 - (100 / (1 + rs))

    def _calculate_macd(self, data: pd.DataFrame) -> pd.Series:
        macd_short_period = self.params.macd1
        macd_long_period = self.params.macd2
        macd_signal_period = self.params.macdsig
        macd = data['close'].ewm(span=macd_short_period, adjust=False).mean() - data['close'].ewm(span=macd_long_period, adjust=False).mean()
        signal = macd.ewm(span=macd_signal_period, adjust=False).mean()
        return macd - signal

    def _calculate_bollinger_bands(self, data: pd.DataFrame) -> (pd.Series, pd.Series):
        bollinger_window = 20
        rolling_mean = data['close'].rolling(window=bollinger_window).mean()
        rolling_std = data['close'].rolling(window=bollinger_window).std()
        return rolling_mean + (rolling_std * 2), rolling_mean - (rolling_std * 2)
