import os
import json
import requests
import logging
from datetime import datetime, timedelta
from dotenv import load_dotenv

# 환경 변수 로드
load_dotenv('C:\\Users\\user\\projects\\st\\.env')

# 로깅 설정
logging.basicConfig(level=logging.INFO)

# settings 딕셔너리 정의
settings = {
    'token_path': os.getenv('TOKEN_PATH'),
    'appkey': os.getenv('MY_APP'),
    'appsecret': os.getenv('MY_SEC'),
    'user_id': os.getenv('USER_ID'),
}

def get_valid_token():
    token_path = settings['token_path']
    try:
        with open(token_path, 'r') as file:
            token_data = json.load(file)
            expiration = datetime.strptime(token_data['expiration'], "%Y-%m-%d %H:%M:%S")
            if expiration > datetime.now():
                logging.info("Using existing token")
                return token_data['access_token']
    except (FileNotFoundError, json.JSONDecodeError):
        pass

    url = "https://openapi.koreainvestment.com:9443/oauth2/tokenP"
    data = {
        'grant_type': 'client_credentials',
        'appkey': settings['appkey'],
        'appsecret': settings['appsecret']
    }
    headers = {
        'Content-Type': 'application/json',
        'User-Agent': os.getenv('MY_AGENT')
    }

    try:
        logging.info("Requesting new token...")
        response = requests.post(url, headers=headers, json=data)
        response.raise_for_status()
        response_data = response.json()
        
        new_token = response_data.get('access_token')
        expires_in = int(response_data.get('expires_in', 3600))
        expiration = datetime.now() + timedelta(seconds=expires_in)

        token_data = {
            'access_token': new_token,
            'expiration': expiration.strftime("%Y-%m-%d %H:%M:%S")
        }
        with open(token_path, 'w') as file:
            json.dump(token_data, file)
        logging.info(f"New token saved. Expiration time: {expiration}")
        return new_token
    except requests.exceptions.RequestException as e:
        logging.error(f"Error refreshing token: {e}")
        return None

def get_api_response(url, headers, params):
    try:
        response = requests.get(url, headers=headers, params=params)
        response.raise_for_status()
        return response.json()
    except requests.exceptions.RequestException as e:
        logging.error(f"Request exception occurred: {e}")
        return None

def get_search_conditions():
    access_token = get_valid_token()
    if not access_token:
        logging.error("Failed to obtain access token.")
        return None

    url = "https://openapi.koreainvestment.com:9443/uapi/domestic-stock/v1/quotations/psearch-title"
    headers = {
        'Content-Type': 'application/json; charset=utf-8',
        'Authorization': f'Bearer {access_token}',
        'appkey': settings['appkey'],
        'appsecret': settings['appsecret'],
        'tr_id': 'HHKST03900300'
    }
    params = {
        'user_id': settings['user_id']
    }

    return get_api_response(url, headers, params)

def get_search_results(seq):
    access_token = get_valid_token()
    if not access_token:
        logging.error("Failed to obtain access token.")
        return None

    url = "https://openapi.koreainvestment.com:9443/uapi/domestic-stock/v1/quotations/psearch-result"
    headers = {
        'Content-Type': 'application/json; charset=utf-8',
        'Authorization': f'Bearer {access_token}',
        'appkey': settings['appkey'],
        'appsecret': settings['appsecret'],
        'tr_id': 'HHKST03900400'
    }
    params = {
        'user_id': settings['user_id'],
        'seq': seq
    }

    response = get_api_response(url, headers, params)
    if response:
        # 주식 종목만 필터링
        stock_results = []
        for result in response.get('output2', []):
            if result.get('daebi') == '2':  # '2'는 주식 종목 코드
                if '인버스' not in result.get('name', '') and 'ETF' not in result.get('name', ''):
                    stock_results.append(result)
        return {'output2': stock_results}
    return None

def test_search_conditions():
    search_conditions = get_search_conditions()
    if not search_conditions:
        logging.error("No search conditions found.")
        return

    for condition in search_conditions.get('output2', []):
        seq = condition.get('seq')
        search_results = get_search_results(seq)
        if not search_results or not isinstance(search_results.get('output2'), list):
            logging.error(f"No search results found for condition sequence {seq}.")
            continue

        for stock in search_results['output2']:
            logging.info(f"Found stock: {stock.get('code')} - {stock.get('name')}")

if __name__ == "__main__":
    test_search_conditions()
