
class StockTradingEnv:
    """
    Stock Trading Environment for reinforcement learning.

    Attributes:
        data (DataFrame): Historical stock prices.
        current_step (int): The current step in the environment.
        done (bool): Whether the environment is done.
        state (Series): The current state of the environment.
    """

    def __init__(self, data):
        """
        Initialize the environment with historical stock prices.

        Parameters:
            data (DataFrame): Historical stock prices.
        """
        if data.empty or 'Close' not in data.columns:
            raise ValueError("Invalid data provided to environment.")

        self.data = data
        self.current_step = 0
        self.done = False
        self.state = None
        self.reset()

    def reset(self):
        """
        Reset the environment to the initial state.

        Returns:
            Series: The initial state of the environment.
        """
        self.current_step = 0
        self.done = False
        if len(self.data) > 0:
            self.state = self.data.iloc[self.current_step]
        else:
            self.done = True
            self.state = pd.Series()
        return self.state

    def step(self, action):
        """
        Apply the action to the environment and return the new state, reward, and done flag.

        Parameters:
            action (int): The action to take (0=hold, 1=buy, 2=sell).

        Returns:
            tuple: A tuple containing the next state (Series), reward (float), and done flag (bool).
        """
        if self.done or self.current_step >= len(self.data) - 1:
            self.done = True
            return self.state, 0, self.done

        self.current_step += 1
        if self.current_step < len(self.data):
            next_state = self.data.iloc[self.current_step]
            reward = self.calculate_reward(action)
            self.state = next_state
        else:
            self.done = True
            next_state = self.state
            reward = 0
        
        return next_state, reward, self.done

    def calculate_reward(self, action):
        """
        Calculate the reward for the given action.

        Parameters:
            action (int): The action taken.

        Returns:
            float: The reward for the action.
        """
        if self.done:
            return 0

        reward = 0
        if action == 1:  # Buy
            reward = self.data.iloc[self.current_step]['Close'] - self.state['Close']
        elif action == 2:  # Sell
            reward = self.state['Close'] - self.data.iloc[self.current_step]['Close']
        return reward
