import os
import json
import pandas as pd
import logging
import websocket
from datetime import datetime, timedelta
import schedule
import time
from settings import settings

logging.basicConfig(level=logging.INFO)

# Define global variables
buy_signals = None
sell_signals = None

def read_stock_data(stock_code):
    file_path = os.path.join(settings['data_path'], f"{stock_code}_data.csv")
    if os.path.exists(file_path):
        try:
            df = pd.read_csv(file_path)
            if not df.empty:
                df = check_and_fix_columns(df)
                return df
            else:
                logging.error(f"Data for stock {stock_code} is empty.")
                return None
        except pd.errors.EmptyDataError:
            logging.error(f"No columns to parse from file {file_path}.")
            return None
    else:
        logging.error(f"Data file for stock {stock_code} not found at {file_path}")
        return None

def check_and_fix_columns(df):
    required_columns = ['Open', 'High', 'Low', 'Close', 'Volume', 'position']
    for column in required_columns:
        if column not in df.columns:
            logging.warning(f"Missing column '{column}' in data, adding default values.")
            df[column] = 0
    return df

def generate_trading_signals(stock_data):
    logging.info(f"Generating trading signals for data:\n{stock_data.head()}")

    if 'Close' not in stock_data.columns:
        logging.error("Data does not contain 'Close' column")
        return stock_data

    stock_data['SMA_20'] = stock_data['Close'].rolling(window=20).mean()
    stock_data['SMA_50'] = stock_data['Close'].rolling(window=50).mean()
    stock_data['signal'] = 0
    stock_data.loc[20:, 'signal'] = (stock_data['SMA_20'][20:] > stock_data['SMA_50'][20:]).astype(int)
    stock_data['position'] = stock_data['signal'].diff()
    
    logging.info(f"Generated signals:\n{stock_data[['Close', 'SMA_20', 'SMA_50', 'signal', 'position']].tail(30)}")
    return stock_data

def get_available_cash():
    balance_json_path = os.path.join(settings['data_path'], 'balance.json')
    if os.path.exists(balance_json_path):
        balance_df = pd.read_json(balance_json_path)
        if 'ord_psbl_cash' in balance_df.columns:
            cash_balance = balance_df['ord_psbl_cash'].astype(int).sum()
            return cash_balance
        else:
            logging.error("ord_psbl_cash column not found in balance data.")
            return 0
    else:
        logging.error(f"Balance file not found at {balance_json_path}")
        return 0

def get_owned_quantity(stock_code):
    balance_json_path = os.path.join(settings['data_path'], 'balance.json')
    if os.path.exists(balance_json_path):
        balance_df = pd.read_json(balance_json_path)
        if stock_code in balance_df['pdno'].values:
            stock_balance = balance_df.loc[balance_df['pdno'] == stock_code, 'hldg_qty'].astype(int).sum()
            return stock_balance
        else:
            logging.error(f"No holdings found for stock code {stock_code}.")
            return 0
    else:
        logging.error(f"Balance file not found at {balance_json_path}")
        return 0

def execute_trades(stock_data):
    global buy_signals, sell_signals  # Ensure these are global
    stock_data = generate_trading_signals(stock_data)  # 전략 생성
    buy_signals = stock_data[stock_data['position'] == 1]
    sell_signals = stock_data[stock_data['position'] == -1]

    logging.info(f"Buy signals:\n{buy_signals}")
    logging.info(f"Sell signals:\n{sell_signals}")

    ws = websocket.WebSocketApp("ws://ops.koreainvestment.com:21000",
                                on_open=on_open,
                                on_message=on_message,
                                on_error=on_error,
                                on_close=on_close)

    ws.run_forever()

def on_message(ws, message):
    logging.info(f"Received message: {message}")

def on_error(ws, error):
    logging.error(f"Error: {error}")

def on_close(ws, close_status_code, close_msg):
    logging.info("Connection closed")

def on_open(ws):
    global buy_signals, sell_signals  # Ensure these are global
    available_cash = get_available_cash()

    if buy_signals is None or sell_signals is None:
        logging.error("Trading signals are not defined.")
        return

    for index, signal in buy_signals.iterrows():
        stock_price = signal['Close']
        quantity = calculate_order_quantity(available_cash, stock_price)
        if quantity > 0:
            order = {
                'type': 'buy',
                'stock_code': signal['stock_code'],
                'price': stock_price,
                'quantity': quantity
            }
            ws.send(json.dumps(order))
            logging.info(f"Buy signal for {signal['stock_code']} at {signal['Close']}, quantity: {quantity}")
            available_cash -= stock_price * quantity

    for index, signal in sell_signals.iterrows():
        stock_price = signal['Close']
        owned_quantity = get_owned_quantity(signal['stock_code'])
        if owned_quantity > 0:
            order = {
                'type': 'sell',
                'stock_code': signal['stock_code'],
                'price': stock_price,
                'quantity': owned_quantity  # 실제 보유 수량으로 판매
            }
            ws.send(json.dumps(order))
            logging.info(f"Sell signal for {signal['stock_code']} at {signal['Close']}, quantity: {owned_quantity}")

def calculate_order_quantity(available_cash, stock_price, risk_factor=0.01, max_portfolio_percent=0.05):
    max_investment = available_cash * risk_factor
    max_quantity = max_investment // stock_price

    max_portfolio_allocation = available_cash * max_portfolio_percent
    max_quantity = min(max_quantity, max_portfolio_allocation // stock_price)

    return max(1, int(max_quantity))  # Ensure at least 1 share is bought if possible

def start_trading():
    stock_codes = [f.split('_')[0] for f in os.listdir(settings['data_path']) if f.endswith('_data.csv')]
    for stock_code in stock_codes:
        stock_data = read_stock_data(stock_code)
        if stock_data is not None:
            execute_trades(stock_data)

def start_training():
    logging.info("Starting training process...")
    # Implement your training logic here
    stock_codes = [f.split('_')[0] for f in os.listdir(settings['data_path']) if f.endswith('_data.csv')]
    for stock_code in stock_codes:
        stock_data = read_stock_data(stock_code)
        if stock_data is not None:
            # Placeholder for training logic
            logging.info(f"Training model for stock {stock_code}")

def wait_until_next_trading_day():
    while True:
        now = datetime.now()
        next_trading_day = now + timedelta(days=1)
        next_trading_day = next_trading_day.replace(hour=9, minute=0, second=0, microsecond=0)
        sleep_time = (next_trading_day - now).total_seconds()
        logging.info(f"Waiting until next trading day: {next_trading_day}")
        time.sleep(sleep_time)
        start_trading()

def is_trading_day():
    today = datetime.today().weekday()
    holidays = ["2024-12-25", "2025-01-01"]  # Example holiday dates, update as needed
    today_str = datetime.today().strftime("%Y-%m-%d")
    if today >= 5 or today_str in holidays:  # Saturday, Sunday, or holiday
        return False
    return True

if __name__ == "__main__":
    schedule.every().day.at("09:00").do(start_trading)
    schedule.every().day.at("15:30").do(start_training)

    while True:
        if is_trading_day():
            schedule.run_pending()
        else:
            logging.info("Today is a holiday. Waiting until the next trading day.")
        time.sleep(60)
