import threading
import time
import torch
import logging
import os
import pandas as pd
import json

from stable_baselines3 import DQN
from stock_trading_env import StockTradingEnv
from settings import settings  # Ensure this is the correct path
from data_collector import collect_condition_data
from trading_logic import execute_trades

# Configure logging
logging.basicConfig(level=logging.INFO, handlers=[
    logging.FileHandler("training.log"),
    logging.StreamHandler()
])

# GPU 확인
device = 'cuda' if torch.cuda.is_available() else 'cpu'
print(f"Using device: {device}")

# 필수 열 목록
REQUIRED_COLUMNS = ['Open', 'High', 'Low', 'Close', 'Volume']

def check_and_fix_columns(df):
    for column in REQUIRED_COLUMNS:
        if column not in df.columns:
            logging.warning(f"Missing column '{column}' in data, adding default values.")
            df[column] = 0
    return df

def read_stock_data(stock_code):
    file_path = os.path.join(settings['data_path'], f"{stock_code}_data.csv")
    if os.path.exists(file_path):
        try:
            df = pd.read_csv(file_path)
            if not df.empty:
                df = check_and_fix_columns(df)
                return df
            else:
                logging.error(f"Data for stock {stock_code} is empty.")
                return None
        except pd.errors.EmptyDataError:
            logging.error(f"No columns to parse from file {file_path}.")
            return None
    else:
        logging.error(f"Data file for stock {stock_code} not found at {file_path}")
        return None

def generate_trading_signals(stock_data):
    # Placeholder for generating trading signals
    # Add your trading signal generation logic here
    return stock_data

def read_balance():
    BALANCE_FILE_PATH = os.path.join(settings['data_path'], 'balance.json')
    if not os.path.exists(BALANCE_FILE_PATH):
        logging.error(f"Balance file not found at {BALANCE_FILE_PATH}")
        return None
    
    try:
        with open(BALANCE_FILE_PATH, 'r', encoding='utf-8') as file:
            balance = json.load(file)
            return balance
    except Exception as e:
        logging.error(f"An error occurred while reading the balance file: {str(e)}")
        return None

def initialize():
    logging.info("Initializing system...")
    # Add your system initialization logic here

# 학습 스레드
def training_thread(data_path=settings['data_path']):
    while True:
        try:
            collect_condition_data()  # 실시간 데이터 수집
        except Exception as e:
            logging.error(f"Error collecting condition data: {e}")
            time.sleep(60)  # 잠시 대기 후 재시도
            continue

        stock_codes = [f.split('_')[0] for f in os.listdir(data_path) if f.endswith('_data.csv')]
        for stock_code in stock_codes:
            file_path = os.path.join(data_path, f"{stock_code}_data.csv")
            if not os.path.exists(file_path):
                logging.error(f"Data file for stock {stock_code} not found at {file_path}")
                continue

            try:
                df = pd.read_csv(file_path)
                if df.empty:
                    logging.error(f"Data for stock {stock_code} is empty. Skipping.")
                    continue
                df = check_and_fix_columns(df)  # 필수 열이 있는지 확인하고 없으면 기본값 추가

                if len(df) < 10:  # 최소 10개 이상의 행이 있어야 학습 가능
                    logging.error(f"Data for stock {stock_code} is insufficient for training. Skipping.")
                    continue

                # 환경을 초기화합니다.
                env = StockTradingEnv(df)
                model_path = f"{settings['model_path']}_{stock_code}.zip"

                # 기존 모델이 있으면 로드하고, 없으면 새로운 모델 생성
                if os.path.exists(model_path):
                    model = DQN.load(model_path, env=env, device=device)
                    logging.info(f"Model for {stock_code} loaded from {model_path}")
                else:
                    model = DQN('MlpPolicy', env, verbose=1, device=device)
                    logging.info(f"New model for {stock_code} created")

                # DQN 에이전트를 학습시킵니다.
                model.learn(total_timesteps=10000, reset_num_timesteps=False, log_interval=10)  # 학습 시간 간격을 줄였습니다.

                # 학습된 모델을 저장합니다.
                model.save(model_path)
                logging.info(f"Model for {stock_code} saved to {model_path}")

            except pd.errors.EmptyDataError:
                logging.error(f"No columns to parse from file {file_path}. Skipping.")
            except Exception as e:
                logging.error(f"An error occurred while processing stock {stock_code}: {str(e)}")

        time.sleep(60)  # 1분 간격으로 모델을 재학습

# 거래 스레드
def trading_thread():
    while True:
        stock_codes = [f.split('_')[0] for f in os.listdir(settings['data_path']) if f.endswith('_data.csv')]
        for stock_code in stock_codes:
            stock_data = read_stock_data(stock_code)
            if stock_data is not None:
                stock_data = generate_trading_signals(stock_data)
                try:
                    execute_trades(stock_data)
                except Exception as e:
                    logging.error(f"Error executing trades for stock {stock_code}: {e}")
            else:
                logging.warning(f"No data found for stock {stock_code}")
        time.sleep(1)  # 1초 간격으로 거래 수행

def start_trading():
    # 스레드 생성
    training = threading.Thread(target=training_thread)
    trading = threading.Thread(target=trading_thread)

    # 스레드 시작
    training.start()
    trading.start()

    # 스레드가 종료될 때까지 대기
    training.join()
    trading.join()

# WebSocket 콜백 함수
def on_close(ws):
    logging.error("Connection to remote host was lost.")

def on_open(ws):
    global buy_signals
    logging.info("Websocket connected")
    # Define or update `buy_signals` if necessary

# 메인 함수
if __name__ == "__main__":
    initialize()
    start_trading()
