import os
import smtplib
from email.mime.text import MIMEText
import yaml
import logging

# Log settings
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

# Set the config file path relative to this script
config_file_path = os.path.join(os.path.dirname(__file__), 'config.yaml')
if not os.path.exists(config_file_path):
    logging.error(f"Config file not found: {config_file_path}")
    raise FileNotFoundError(f"Config file not found: {config_file_path}")

with open(config_file_path, 'r', encoding='utf-8') as file:
    config = yaml.safe_load(file)

email_config = config['email_config']

def send_email(subject, body, to_email):
    try:
        msg = MIMEText(body)
        msg['Subject'] = subject
        msg['From'] = email_config['from_email']
        msg['To'] = to_email

        with smtplib.SMTP(email_config['smtp_server'], email_config['smtp_port']) as server:
            server.starttls()
            server.login(email_config['from_email'], email_config['password'])
            server.sendmail(email_config['from_email'], to_email, msg.as_string())
        logging.info(f"Email sent to {to_email}")
    except Exception as e:
        logging.error(f"Failed to send email: {e}")
        raise

def notify_performance(performance_metrics):
    subject = "Trading System Performance Alert"
    body = f"Performance Metrics:\n\n{performance_metrics}"
    to_email = email_config['to_email']
    send_email(subject, body, to_email)

def monitor_performance(data):
    threshold_profit = 0.05  # Example threshold for sending alerts
    cumulative_returns = data['cumulative_returns'].iloc[-1]
    if cumulative_returns > threshold_profit:
        performance_metrics = analyze_performance(data)
        notify_performance(performance_metrics)

def analyze_performance(data):
    returns = data['strategy_returns'].dropna()
    cumulative_returns = data['cumulative_returns'].iloc[-1]
    annual_return = returns.mean() * 252
    annual_volatility = returns.std() * np.sqrt(252)
    sharpe_ratio = annual_return / annual_volatility
    max_drawdown = calculate_max_drawdown(data['cumulative_returns'])
    
    performance = {
        'Cumulative Returns': cumulative_returns,
        'Annual Return': annual_return,
        'Annual Volatility': annual_volatility,
        'Sharpe Ratio': sharpe_ratio,
        'Max Drawdown': max_drawdown
    }
    
    return performance

def calculate_max_drawdown(series):
    cumulative = series.cummax()
    drawdown = series / cumulative - 1
    max_drawdown = drawdown.min()
    return max_drawdown
