from flask import Flask, request, jsonify, render_template
from flask_socketio import SocketIO
import threading
import logging
from datetime import datetime
import train_agent  # 수정: train_agent를 import

logging.basicConfig(level=logging.INFO)
app = Flask(__name__)
socketio = SocketIO(app)
trading_thread = None
trading_active = threading.Event()

@app.route('/')
def index():
    return render_template('index.html')

@app.route('/initialize', methods=['GET'])
def init_system():
    try:
        train_agent.initialize()  # 수정: train_agent의 initialize 호출
        logging.info("시스템 초기화가 완료되었습니다.")
        return jsonify({"message": "시스템 초기화가 완료되었습니다."}), 200
    except Exception as e:
        logging.error(f"시스템 초기화 중 오류가 발생했습니다: {e}")
        return jsonify({"message": f"시스템 초기화 중 오류가 발생했습니다: {e}"}), 500

@app.route('/start_trading', methods=['POST'])
def start_trading():
    global trading_thread, trading_active
    if trading_thread is None or not trading_thread.is_alive():
        trading_active.set()
        trading_thread = threading.Thread(target=train_agent.start_trading)  # 수정: train_agent의 start_trading 호출
        trading_thread.start()
        logging.info("거래가 성공적으로 시작되었습니다.")
        return jsonify({"message": "거래가 성공적으로 시작되었습니다."}), 200
    else:
        logging.warning("이미 거래가 실행 중입니다.")
        return jsonify({"message": "이미 거래가 실행 중입니다."}), 400

@app.route('/stop_trading', methods=['POST'])
def stop_trading():
    global trading_active
    try:
        train_agent.stop_trading()  # 수정: train_agent의 stop_trading 호출
        trading_active.clear()
        if trading_thread is not None and trading_thread.is_alive():
            trading_thread.join()
        logging.info("거래가 성공적으로 중지되었습니다.")
        return jsonify({"message": "거래가 성공적으로 중지되었습니다."}), 200
    except Exception as e:
        logging.error(f"거래 중지 중 오류가 발생했습니다: {e}")
        return jsonify({"message": f"거래 중지 중 오류가 발생했습니다: {e}"}), 500

@app.route('/analyze', methods=['POST'])
def analyze():
    try:
        stocks = train_agent.detect_stocks()  # 수정: train_agent의 detect_stocks 호출
        if stocks:
            logging.info(f"Detected stocks: {stocks}")
            train_agent.collect_data(stocks)  # 수정: train_agent의 collect_data 호출
            logging.info("Data collection completed.")
            train_agent.run_analysis()  # 수정: train_agent의 run_analysis 호출
            logging.info("분석 및 학습이 성공적으로 시작되었습니다.")
            return jsonify({"message": "분석 및 학습이 성공적으로 시작되었습니다."}), 200
        else:
            logging.warning("분석 및 학습을 위한 종목이 없습니다.")
            return jsonify({"message": "분석 및 학습을 위한 종목이 없습니다."}), 400
    except Exception as e:
        logging.error(f"분석 및 학습 중 오류가 발생했습니다: {e}")
        return jsonify({"message": f"분석 및 학습 중 오류가 발생했습니다: {e}"}), 500

if __name__ == "__main__":
    try:
        logging.info("Starting Flask server")
        socketio.run(app, host='0.0.0.0', port=8000, debug=True)
    except Exception as e:
        logging.error(f"Error starting Flask server: {e}")
    finally:
        if trading_thread is not None and trading_thread.is_alive():
            trading_active.clear()
            trading_thread.join()
        logging.info("Flask server stopped")
