import os
import websocket
import json
import pandas as pd
import numpy as np
from collections import deque
from datetime import datetime
from .config_manager import ConfigManager, parse_mst_file
import talib as ta
import logging

# Ensure dependencies are installed
try:
    import websockets
except ImportError:
    os.system('python3 -m pip install websocket-client')

# Add logging configuration
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

class BasicPlan:
    def __init__(self, stock_code, window=20):
        self._stock_code = stock_code
        self._queue = deque(maxlen=window)
        self._prev_ma = None

    def push(self, value):
        self._queue.append(value)
        ma = sum(self._queue) / len(self._queue)
        diff = ma - self._prev_ma if self._prev_ma is not None else None
        self._prev_ma = ma
        logging.info(f"{self._stock_code} - Value: {value}, MA: {ma}, Diff: {diff}")

class RSI_ST:
    def __init__(self, stock_code, window=21):
        self._stock_code = stock_code
        self._queue = deque(maxlen=window)
        self.rsi_period = window

    def eval(self, prices):
        np_closes = np.array(prices, dtype=np.float64)
        rsi = ta.RSI(np_closes, self.rsi_period)
        last_rsi = rsi[-1]

        if last_rsi < 30:
            logging.info(f"{self._stock_code} [BUY] RSI: {last_rsi}")
        elif 30 <= last_rsi < 70:
            logging.info(f"{self._stock_code} [HOLD] RSI: {last_rsi}")
        elif last_rsi >= 70:
            logging.info(f"{self._stock_code} [SELL] RSI: {last_rsi}")
        else:
            logging.info(f"{self._stock_code} Invalid RSI value")

class WebSocketClient:
    def __init__(self, url, kosdaq_data, kospi_data):
        self.url = url
        self.kosdaq_data = kosdaq_data
        self.kospi_data = kospi_data
        self.ws = None
        self.contract_sub_df = {code: pd.DataFrame() for code in kosdaq_data + kospi_data}
        self.tr_plans = {code: BasicPlan(code) for code in kosdaq_data + kospi_data}

    def on_message(self, ws, message):
        logging.info(f"Received message: {message}")
        data = json.loads(message)
        self._dparse(data)

    def on_error(self, ws, error):
        logging.error(f"WebSocket error: {error}")

    def on_close(self, ws, close_status_code, close_msg):
        logging.info(f"WebSocket closed: {close_msg}")

    def on_open(self, ws):
        logging.info("WebSocket connection opened")
        for code in self.kosdaq_data + self.kospi_data:
            self.subscribe(ws, code)

    def subscribe(self, ws, stock_code):
        message = json.dumps({
            "type": "subscribe",
            "stock_code": stock_code
        })
        ws.send(message)

    def _dparse(self, data):
        stock_code = data['stock_code']
        price = data['price']
        self.tr_plans[stock_code].push(price)
        self.contract_sub_df[stock_code] = self.contract_sub_df[stock_code].append(data, ignore_index=True)

        prices = self.contract_sub_df[stock_code]['price'].astype(float).tolist()
        rsi_strategy = RSI_ST(stock_code)
        rsi_strategy.eval(prices)

    def run(self):
        websocket.enableTrace(True)
        self.ws = websocket.WebSocketApp(
            self.url,
            on_message=self.on_message,
            on_error=self.on_error,
            on_close=self.on_close,
            on_open=self.on_open
        )
        self.ws.run_forever()

# Initialize ConfigManager
base_dir = os.path.join('C:\\Users\\user\\KIS\\config')
config_path = os.path.join(base_dir, 'config.json')
env_path = os.path.join(base_dir, '.env')
config_manager = ConfigManager(config_path, env_path)

# Parse the MST files
kosdaq_path = os.path.join(base_dir, 'kosdaq_code.mst')
kospi_path = os.path.join(base_dir, 'kospi_code.mst')
kosdaq_data = parse_mst_file(kosdaq_path)
kospi_data = parse_mst_file(kospi_path)

# Retrieve WebSocket URL from configuration
ws_url = config_manager.get('websocket.url', 'ws://localhost:8000')

# Initialize and run WebSocket client
ws_client = WebSocketClient(ws_url, kosdaq_data, kospi_data)
ws_client.run()
